﻿using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using RazorPagesDownload.MyCode;
using System.IO;
using System.Linq;
using WebMVCDownload.Models;

//====================================================
// Описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================

namespace WebMVCDownload.Controllers
{
    public class DownloadController : Controller
    {
        private readonly ILogger<DownloadController> _logger;
        private readonly IWebHostEnvironment _environment;


        public DownloadController(ILogger<DownloadController> logger, IWebHostEnvironment env)
        {
            _logger = logger;
            _environment = env;
        }


        public IActionResult Index()
        {

            return View();
        }

        public IActionResult PrivateFiles(string file, string mode)
        {
            // Путь к закрытому каталогу.
            string pathDirectory = Path.Combine(MyConstants.PrivateDownloadDirectory);

            //string pathDirectory = Path.Combine(MyConstants.CustomDownloadDirectory);


            return SelectiveResult(file, mode, pathDirectory);
        }

        public IActionResult PublicFiles(string file, string mode)
        {
            // Путь к открытому каталогу.
            string pathDirectory = Path.Combine(_environment.WebRootPath, MyConstants.PublicDownloadDirectory);


            return SelectiveResult(file, mode, pathDirectory);
        }


        #region Методы вычисления типа возврата

        [NonAction]
        private IActionResult SelectiveResult(string file, string mode, string pathDirectory)
        {
            if (file == null)
            {
                var model = new Download();

                // Заполняем список данными файлов для показа на странице.
                var di = new DirectoryInfo(pathDirectory);

                // При первом извлечении свойств FileInfo вызывает Refresh метод и кэширует сведения о файле.
                // При последующих вызовах необходимо вызвать, Refresh чтобы получить последнюю копию информации.
                model.ListFiles = di.GetFiles().ToList();

                return View(model);
            }
            else
            {
                var fi = new FileInfo(Path.Combine(pathDirectory, file));
                fi.Refresh();

                if (fi.Exists == true)
                {
                    return mode switch
                    {
                        "virtual" => VirtualDownload(fi),
                        "stream" => StreamDownload(fi),
                        "bytes" => BytesDownload(fi),
                        _ => new EmptyResult(),
                    };

                }
                else
                {
                    // Обработка ситуации отсутствия файла.

                    return new EmptyResult();
                }
            }
        }

        #endregion



        #region Способы отдачи файлов пользователю

        [NonAction]
        private IActionResult VirtualDownload(FileInfo fi)
        {
            var contentType = MyUtility.ContentTypes(fi.Extension);

            // Виртуальный путь относительно адреса хоста, 
            // например: полный адрес файла http://host.ru/directory/file
            // относительный /directory/file
            string virtualPath = MyConstants.PublicDownloadDirectory + "//" + fi.Name;

            return File(virtualPath, contentType, fi.Name);
        }

        

        [NonAction]
        private ActionResult BytesDownload(FileInfo fi)
        {
            // Получаем байты файла.
            var bytes = System.IO.File.ReadAllBytes(fi.FullName);

            var contentType = MyUtility.ContentTypes(fi.Extension);

            return new FileContentResult(bytes, contentType)
            {
                FileDownloadName = fi.Name
            };
        }


        [NonAction]
        private IActionResult StreamDownload(FileInfo fi)
        {
            // Открываем поток.
            var stream = fi.OpenRead(); //System.IO.File.OpenRead(path);

            var contentType = MyUtility.ContentTypes(fi.Extension);

            // 1 способ
            // return File(stream, content_type, file);

            // 2 способ
            return new FileStreamResult(stream, contentType)
            {
                FileDownloadName = fi.Name
            };
        }

        #endregion
    }

    class ActionDispatcher
    {

    }
}
